#ifndef OFFSETCONTROL_H
#define OFFSETCONTROL_H

#include "StatusData.h"



/*!
 * \class OffsetControl OffsetControl.h
 *
 * \brief Class that performs the operations for calibrating and correcting the
 * offset that EEG/ECG signal may have.
 */
class OffsetControl
{
public:


    /*!
     * Default constructor.
     */
    OffsetControl (unsigned int numberOfChannels);

    /*!
     * Constructor that provides the upper and lower limits to be monitored.
     *
     * \param upperLimit The class ask for correcting the offset the signal
     * goes above this value.
     *
     * \param lowerLimit The class ask for correcting the offset the signal
     * goes below this value.
     */
    OffsetControl (unsigned int numberOfChannels, int uL, int lL);

    /*!
     * Default destructor.
     */
    ~OffsetControl ();

    /*!
    * It sets the upper and lower limits to be monitored.
    *
    * \param upperLimit The class ask for correcting the offset the signal
    * goes above this value.
    *
    * \param lowerLimit The class ask for correcting the offset the signal
    * goes below this value.
    */
    void setLimits (int uL, int lL);

    /*!
     * Access to the upper limit to be monitored
     *
     * \return Value of the upper limit that the ECG/EEG signal shall not cross.
     */
    int getUpperLimit ();

    /*!
     * Access to the lower limit to be monitored
     *
     * \return Value of the lower limit that the ECG/EEG signal shall not cross.
     */
    int getLowerLimit ();

    /*!
     * Have an offset change has been send and we are waiting the reply from enobio. If yes then return true.
     *
     */
    bool getWaitingToggle ();

    /*!
     * Set the state of the system according to whether an offset change has been successful or not.
     *
     */
    void setWaitingToggle (bool boo);

    /*!
     * Set which channels will be calibrated or not
     */
    void calibrateChannel (unsigned char channelIndex, bool isCalibrated);

    /*!
     * Update the average of each channel
     */
    void update (int newValues[]);

    /*!
     * Check the values of the channels and return the channel that needs to be calibrated and to which direction
     */
    char check ();

    /*!
     * Set the step size of each offset change
     *
     */
    void setStepSize (unsigned char step);

    /*!
     * Get the offset value for each of the channels. This is a value that
     * ranges from 0 to 1024 representing voltages from 0 to 2.45.
     *
     */
    unsigned short getOffset (unsigned char index);


    /*!
     * Get the status of the channel with the index
     */

    StatusData::statusCode getStatus (unsigned char index);

    /*!
     * Set the status of the channel with the index
     */

    void setStatus (unsigned char index, StatusData::statusCode code);



    /*!
     * This returns the index of the channel that is currently being
     * monitored.
     *
     * \return The index of the channel (0,1,2,3)
     */
    unsigned char channelCurrentlyChecking ();

    /*!
     * Start or stop monitoring a specific channel
     */
    void setCalibrateChannel(unsigned char index, bool isCalibrating);

    /*!
     * Check is a specific channel is being monitored or not
     */
    bool getCalibrateChannel(unsigned char index);

    /*!
     * Get the average value of a specific channel
     */
    int getAverage(unsigned char index);

    /**
     * Offset control will move to the next channel
     */
    void nextChannel();

    /**
     * Reset some variables everytime the streaming is stopped
     */
    void reset();

private:

    /*!
     * The number of channels in the system. Currently set to 4
     */
    unsigned int _numberOfChannels;

    /*!
     * The minimum accepted value
     */
    int _lowerLimit;

    /*!
     * The maximum accepted value
     */
    int _upperLimit;

    /*!
     * This boolean shows the state of calibration. If it true then there is a
     * pending offset change
     */
    bool _waitingToggle;

    /*!
     * Which channels are selected to calibrate
     */
    bool * _isChannelCalibrating;

    /*!
     * The average channel values. The algorithm to correct the offset uses this value to monitor the state
     * of channel
     */
    int * _average;

    /*!
     * The channel that is being monitored at the moment
     */
    unsigned char _currentlyChecking;

    /*!
     * The actual value of the reference to be sent for each channel. This is a value from 0 to 1024.
     */
    unsigned short * _offsets;

    /*!
     * This is the actual status of each of the channels.
     * A channel at any point in time could either be calibrated, in the process of calibrating
     * or not selected to be calibrated.
     */
    StatusData::statusCode * _status;

    /*!
     * The value that is added or subtracted to the current offset
     */
    unsigned char _stepSize;

    /*!
     * This just initialises the local variables to a default value
     */
    void _iniPrivateData(unsigned int numberOfChannels);



};

#endif // OFFSETCONTROL_H
