#ifndef FTDIDEVICE_H
#define FTDIDEVICE_H

#include "QLibrary"
#include "HWDevice.h"
#include "FTD2XX.H"
#include "PropertyCollection.h"


/*!
 * \class FTDIDevice FTDIDevice.h
 *
 * \brief Implementation of the FTDI hardware device
 *
 */
class FTDIDevice : HWDevice
{
public:
    /*!
     * Default constructor
     */
    FTDIDevice ();

    /*!
     * Default destructor
     */
    virtual ~FTDIDevice ();

    virtual HWErrType open ();

    virtual HWErrType close ();

    virtual HWErrType reset ();

    virtual bool configure ();

    virtual int read (unsigned char * buffer, unsigned long numBytes);

    virtual int write (unsigned char * buffer, unsigned long numBytes);

    virtual HWErrType getLastError ();

    HWErrType open (DWORD index);

    /*!
     * \enum FTDIDevice::errType
     *
     * Description of the possible error types.
     */
    enum errType
    {
        ERR_NO_ERROR = 0,
        ERR_LIBRARY_NOT_LOADED,
        ERR_LIBRARY_SIMBOL_NOT_LOADED,
        ERR_RETRIEVING_LIST_DEVICES,
        ERR_DEVICE_NOT_CONNECTED,
        ERR_CLOSING_DEVICE,
        ERR_RESETING_DEVICE,
        ERR_DEVICE_NOT_OPENED,
        ERR_READING_DEVICE,
        ERR_WRITING_DEVICE,
        ERR_CONFIGURING_DEVICE,
        ERR_CONFIGURATION_VALUE
    };

    enum wordLengthValue
    {
        WORD_LENGTH_8 = FT_BITS_8,
        WORD_LENGTH_7 = FT_BITS_7,
        WORD_LENGTH_6 = FT_BITS_6,
        WORD_LENGTH_5 = FT_BITS_5,
    };

    enum stopBitsValue
    {
        STOP_BITS_1 = FT_STOP_BITS_1,
        STOP_BITS_1_5 = FT_STOP_BITS_1_5,
        STOP_BITS_2 = FT_STOP_BITS_2,
    };

    enum parityValues
    {
        PARITY_NONE = FT_PARITY_NONE,
        PARITY_ODD = FT_PARITY_ODD,
        PARITY_EVEN = FT_PARITY_EVEN,
        PARITY_MARK = FT_PARITY_MARK,
        PARITY_SPACE = FT_PARITY_SPACE,
    };

    DWORD getNumDevices ();

    DWORD getPendingBytes ();

    bool setBaudRate (unsigned int baudRate);

    unsigned int getBaudRate ();

    bool setWordLength(unsigned int wordLength);

    unsigned int getWordLength ();

    bool setStopBits (unsigned int stopBits);

    unsigned int getStopBits ();

    bool setParity (unsigned int parity);

    unsigned int getParity ();

    bool setLatencyTimer (unsigned int latencyTimer);

    unsigned int getLatencyTimer ();

    bool setReadTimeout (unsigned int timeout);

    unsigned int getReadTimeout ();

private:
    /*!
     * It internally sets the error variable with a new value.
     *
     * \param error Error type from the errType enumeration.
     */
    void _setError (errType error);

    /*!
     * It loads the FTDI dll and initializes the pointer to the dll functions.
     */
    void _loadLibrary ();

    /*!
     * To Be Commented
     */
    HWErrType _findAndOpenDevice ();

    /*!
     * To Be Commented
     */
    unsigned int _baudRate;

    /*!
     * To Be Commented
     */
    wordLengthValue _wordLength;

    /*!
     * To Be Commented
     */
    stopBitsValue _stopBits;

    /*!
     * To Be Commented
     */
    parityValues _parity;

    /*!
     * To Be Commented
     */
    unsigned int _readTimeout;

    /*!
     * To Be Commented
     */
    unsigned int _latencyTimer;

    /*!
     * It returns the device open status.
     *
     * \return True if the device is already opened, false otherwise.
     */
    bool _isOpen ();

    /*!
     * It returns wheter the open device is still connected.
     *
     * \return True if the devicei is already open and connected, false
     * otherwise.
     */
    bool _isConnected();

    /*!
     * Pointer to the Qt object that handles an externall dll and its symbols.
     */
    QLibrary * _library;

    /*!
     * The last error that might happen is stored in that variable.
     */
    errType _lastError;

    /*!
     * Handle to the FTDI hardware device
     */
    FT_HANDLE _ftHandle;

    /*!
     * Pointers to the dll functions
     */
    typedef FT_STATUS (WINAPI *PtrToOpen)(int, FT_HANDLE *);
    PtrToOpen _ptrToOpen;
    typedef FT_STATUS (WINAPI *PtrToListDevices)(PVOID, PVOID, DWORD);
    PtrToListDevices _ptrToListDevices;
    typedef FT_STATUS (WINAPI *PtrToClose)(FT_HANDLE);
    PtrToClose _ptrToClose;
    typedef FT_STATUS (WINAPI *PtrToRead)(FT_HANDLE, LPVOID, DWORD, LPDWORD);
    PtrToRead _ptrToRead;
    typedef FT_STATUS (WINAPI *PtrToWrite)(FT_HANDLE, LPVOID, DWORD,
                                              LPDWORD);
    PtrToWrite _ptrToWrite;
    typedef FT_STATUS (WINAPI *PtrToResetDevice)(FT_HANDLE);
    PtrToResetDevice _ptrToResetDevice;
    typedef FT_STATUS (WINAPI *PtrToGetStatus)(FT_HANDLE, LPDWORD, LPDWORD,
                                                  LPDWORD);
    PtrToGetStatus _ptrToGetStatus;
    typedef FT_STATUS (WINAPI *PtrToPurge)(FT_HANDLE, ULONG);
    PtrToPurge _ptrToPurge;
    typedef FT_STATUS (WINAPI *PtrToSetTimeouts)(FT_HANDLE, ULONG, ULONG);
    PtrToSetTimeouts _ptrToSetTimeouts;
    typedef FT_STATUS (WINAPI *PtrToGetQueueStatus)(FT_HANDLE, LPDWORD);
    PtrToGetQueueStatus _ptrToGetQueueStatus;
    typedef FT_STATUS (WINAPI *PtrToSetBaudRate)(FT_HANDLE, DWORD);
    PtrToSetBaudRate _ptrToSetBaudRate;
    typedef FT_STATUS (WINAPI *PtrToSetDataCharacteristics)(FT_HANDLE,
                                                               UCHAR,
                                                               UCHAR,
                                                               UCHAR);
    PtrToSetDataCharacteristics _ptrToSetDataCharacteristics;
    typedef FT_STATUS (WINAPI *PtrToSetLatencyTimer)(FT_HANDLE, UCHAR);
    PtrToSetLatencyTimer _ptrToSetLatencyTimer;

};

#endif // FTDIDEVICE_H
