#ifndef IDATAPROCESSOR_H
#define IDATAPROCESSOR_H

#include "IDataConsumer.h"
#include "IDataProducer.h"


/*!
 * \typedef const int ID
 *
 * \brief Identification of the
 */
typedef const int ID;

/*!
 * \class IDataProcessor IDataProcessor.h
 *
 * \brief Base class that allows to use the IDataConsumer and IDataProducer
 * interfaces in order to process incoming data and pass it to other
 * DataProcessors that might be attached.
 *
 */
class IDataProcessor
{
public:
    /*!
     * Virtual destructor
     */
    virtual ~IDataProcessor () { }

    /*!
     * To obtain a data consumer of the DataProcessor
     *
     * \param id Identification of the data consumer to be obtained
     *
     * \return The requested IDataConsumer object
     */
    virtual IDataConsumer * getDataConsumer (ID id) = 0;

    /*!
     * To obtain a data producer of the DataProcessor
     *
     * \param id Identification of the data producer to be obtained
     *
     * \return The requested IDataProducer object
     */
    virtual IDataProducer * getDataProducer (ID id) = 0;

    /*!
     * It registers an IDataConsumer from another DataProcessor to an
     * IDataConsumer of the current DataProcessor
     *
     * \param id Identification of the data producer to which the Idata
     * consumer is going to be attached
     *
     * \param dataConsumer data consumer that is attached to the data producer
     * identified by the id parameter.
     */
    virtual bool registerConsumer (ID id, IDataConsumer& dataConsumer) = 0;

    /*!
     * It deregisters a previous registered data consumer.
     *
     * \param id Identification of the data producer to which the data consumer
     * that is going to be deregestired is attached.
     *
     * \param dataConsumer Data consumer that is deregisterd.
     */
    virtual bool deregisterConsumer (ID id,
                                     const IDataConsumer& dataConsumer) = 0;

    /*!
     * It connects two given processors through registering the consumer of the
     * second processor in the producer of the first one.
     *
     * \param processor1 Reference to the processor which producer is going to
     * have the consumer registration.
     *
     * \param idProducer Identification of the producer that processor1 has.
     *
     * \param processor2 Reference to a the processor which consumer is going
     * to be registered in the producer.
     *
     * \return true if the registration between the producer and the consumer
     * is succesfully, false otherwise.
     */
    static bool connectProcessors (IDataProcessor & processor1,
                                   ID idProducer,
                                   IDataProcessor & processor2,
                                   ID idConsumer)
    {
        IDataProducer * producer = processor1.getDataProducer(idProducer);
        if (producer == NULL)
        {
            return false;
        }
        IDataConsumer * consumer = processor2.getDataConsumer(idConsumer);
        if (consumer == NULL)
        {
            return false;
        }
        producer->addDataConsumerEndPoint(*consumer);
        return true;
    }

    static bool disconnectProcessors (IDataProcessor & processor1,
                                      ID idProducer,
                                      IDataProcessor & processor2,
                                      ID idConsumer)
    {
        IDataProducer * producer = processor1.getDataProducer(idProducer);
        if (producer == NULL)
        {
            return false;
        }
        IDataConsumer * consumer = processor2.getDataConsumer(idConsumer);
        if (consumer == NULL)
        {
            return false;
        }
        producer->removeDataConsumerEndPoint(*consumer);
        return true;
    }
};

#endif // IDATAPROCESSOR_H
